     /* status 	 */
#define FAILED 1
#define PASSED 2
#define PERIOD 3

     /* uClass  */
#define NORMAL 1
#define TRIAL 2
#define EXEMPT 3
#define DELETED 4

     /* checktype */
#define NA 0
#define DAYUP 1
#define WKUP 2
#define MONTHUP 3
#define ALLUP 4


#ifndef MAXPATHLEN
#define MAXPATHLEN 256
#endif

#define DELSIZE 256

     /* Max Users Per Group */
#define MAX_UPG 80

#define DEBUG
#define VERSION "0.20"
#define LINELEN 400
#define DELETED_FLAG "6"

#define HALFDAY 43200
#define DAYSEC 86400
#define WEEKSEC 604800
#define MONTHSEC 2592000

#define LONGLIST 1
#define SHORTLIST 2

#define DONTDEL 1
#define NORMALDEL 2
#define FORCEDEL 3

#define MALLOCERROR "ERROR: Could not allocate memory (not enough memory)!"

/* User structure */
struct scrUser {
	char name[23];		/* username         */
	char flags[50];		/* User's flags     */
	char groups[256];	/* User's groups    */
	long dayup;		/* Daily uploads    */
	long wkup;		/* Weekly uploads   */
	long monthup;		/* Monthly uploads  */
	long allup;		/* All uploads      */
	long minmb;		/* Min amount of megs to upload */
	long upload;		/* Amount of megs uploaded */
	int uClass;		/* NORMAL, TRIAL, EXEMPT  */
	int status;		/* PASSED, FAILED, PERIOD  */
	int checktype;		/* WKUP, DAYUP, MONTHUP, ALLUP  */
	int deleted;		/* 0=not, 1=deletd	*/
	time_t addedon; 	/* linuxtime of added */
	struct scrUser *next;	/* points to the next user on the list */
};

/* Group structure */
struct scrGroup {
	char name[24];
	struct scrUser *users[MAX_UPG];
	int nou;
	long monthup;
	long wkup;
	long dayup;
	long allup;
	long minmb;
	long *uploaded;
	int deleted;
	int checktype;
	int status;
};

typedef struct scrUser scrUser;
typedef struct scrGroup scrGroup;
typedef struct dirent dirent;

struct statistics {
	int total_exempt;
	int total_trial;
	int total_normal;

	int total_users;
	int total_deleted;

	int total_passed;
	int total_failed;
	int total_period;
};

void terminated(void);

/* Linked list related functions */
int free_item(scrUser *item);
scrUser *alloc_item(void);
int link_item(scrUser *anchor, scrUser *item);
scrUser *list_init(void);
void listfree(scrUser *anchor);
scrUser *last_entry(scrUser *anchor);
scrUser *find_by_name(const char *name,const scrUser *anchor);
int list_is_empty(const scrUser *anchor);

/* User management related functions */
void readuser(const char *sUserName);
void handleuser(scrUser *currentUser);
long totalup(const char *line);
void fillupload(const char *checktype, scrUser *currentUser);
void deluser(scrUser *currentUser);
void putuserin(char *username, char *groupname);
int isDelTime(int checktype);
#ifdef ENABLE_WARNING_MSG
void sendmsg(scrUser *currentUser);
#endif
#ifdef GROUP_TRIAL_FILE
void delgroup(scrGroup *delgrp);
#endif
void getexemptgroups(void);

/* User struct related functions */
long getperiod(scrUser *currentUser);
int IsUserInGroup(const char *usergroup, const char *groups);

/* output functions */
void printlist(int filter_class, int filter_status, int listtype);
void printuser(const char *user);
void botprint(const char *user);
void group_botprint(const char *group);
void printusage(char *progpath);
void exiterror(const char *strerror);
#ifdef GROUP_TRIAL_FILE
void g_botprint(char *group);
#endif

