/******************************************************************************
* Filename: jout.c
* Description: Output functions of JeTrial-C
* Author: [Jedi] (jedi_il@hotmail.com)
* Copyright 2002 [Jedi]
* See COPYING for the GPL GNU
*
*    This file is part of JeTrial-C.
*
*    JeTrial-C is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    JeTrial-C is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Foobar; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/

#include <stdio.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/param.h>

#include "../header/strlib.h"
#include "../config/jtconf.h"
#include "../header/proto.h"

extern scrUser *userPrime;
extern char tstr[512], wk_deleted[DELSIZE], mn_deleted[DELSIZE], dy_deleted[DELSIZE],
            nogroup[], *sExemptGroups;
extern const char *getchecktype[], *getclass[], *getstatus[];
extern const int d_mon[];

extern struct tm *now;
extern struct statistics stats;

#ifdef GROUP_TRIAL_FILE
	extern int nTG;	/* Number of Trial Groups */
	extern char grp_deleted[DELSIZE];
	extern scrGroup *grpArray;
#endif
/**********************************************************
*  Function name: printlist()
*  Arguments: 	filter_class - NORMAL, EXEMPT, TRIAL, 0(all)
*		filter_status - PASSED, FAILED, PERIOD, 0(all)
*		listtype - LONGLIST, SHORTLIST
*  Returns: None
*  Description: 
*    The function will print to strout, a short or a long list
*    of the users, filtered by class and/or by status.           
*  Modified: 14/7/02
*************************************************************/

void printlist(int filter_class, int filter_status, int listtype) {
	scrUser *puser;
	char delch;
	puser=userPrime->next;
	if (listtype==LONGLIST) {
		printf("  ...:: JeTrial-C v%s by [Jedi] ::....................\n",VERSION);
		printf(" / %-12s | %-7s | %-7s | %-6s | %-6s \\\n","User","Class","Checked",
			"Status","Uploaded");
		printf(",`------------------------------------------------------',\n");
	}
	while(puser!=NULL) {
		if ( ((puser->uClass==filter_class) || (filter_class==0)) 
		  && ((puser->status==filter_status) || (filter_status==0)) ) {
			if (puser->deleted) delch='*';
			else delch=' ';

			if (listtype==LONGLIST) {
				printf("|%c%-13s | %-7s | %-7s | %-6s | %6ld mB |\n",
				delch, puser->name, getclass[puser->uClass] , getchecktype[puser->checktype],
				getstatus[puser->status], puser->upload);
			} else if (listtype==SHORTLIST) {
				printf("%s ",puser->name);
			}
		}
		puser=puser->next;
	}
	if (listtype==LONGLIST) {
		printf("`--------------------------------------------------------'\n");
		printf("Statistics....\n");
		printf(" Exempt: %-6d Passed: %d\n Trial : %-6d Failed: %d\n Normal: %-6d Period: %d\n Total : %-6d Deleted: %d\n",
		stats.total_exempt, stats.total_passed, stats.total_trial, stats.total_failed,
		stats.total_normal, stats.total_period, stats.total_users ,stats.total_deleted);
	}
}	

/**********************************************************
*  Function name: printuser()
*  Arguments: (string) user - user name.
*  Returns: None
*  Description:
*    The function will output to stdout a card of user's 
*    info, it'll find the user structure by his name.
*  Modified: 14/7/02
*************************************************************/

void printuser(const char *user) {
	scrUser *currentUser;
	if ((currentUser=find_by_name(user,userPrime))==NULL) {
		printf("Error: user '%s' not found!\n",user);
		return;
	}
	printf("\
 ,----------------------------,\n\
| Username %18s |\n\
| Groups %20s |\n\
| Flags %21s |\n\
| DayUp %19ldMb |\n\
| WeekUp %18ldMb |\n\
| MonthUp %17ldMb |\n\
| AllUp %19ldMb |\n\
| Class %21s |\n\
| Status %20s |\n\
| Check Type %16s |\n\
| MinMb %19ldMb |\n\
 `----------------------------'\n",
	currentUser->name, currentUser->groups, currentUser->flags,
	currentUser->dayup, currentUser->wkup, currentUser->monthup,
	currentUser->allup, getclass[currentUser->uClass], 
	getstatus[currentUser->status],	
	getchecktype[currentUser->checktype],currentUser->minmb);
}

/**********************************************************
*  Function name: botprint()
*  Arguments: (string) user - user name.
*  Returns: None
*  Description: 
*    The function will output to stdout a line, which was
*    defined on jtconf.h according to the user status,
*    this line should be used in conjunction with the tcl
*    script (passedbot.tcl) for an eggdrop (see README).
*  Modified: 30/9/02
*************************************************************/

void botprint(const char *user) {
	scrUser *currentUser;
	/*struct tm *tms;
	time_t temp_t;*/
	char *group;
	long minquota=0, needs=0, uploaded=0;
	int daysleft=0, dayspassed=0;

	if ((currentUser=find_by_name(user,userPrime))==NULL) {
		printf(BOT_NOTFOUND);
		return;
	}
	

	/* dayspassed contains the days that have passed since the user was added.
	   its done by subtructing the date of which the user was added from now 
	   (all done in seconds) and dividing by 86400 (which are the number 
	   of seconds in 1 day. */
	dayspassed=(int)((time(NULL) - currentUser->addedon)/86400);

	/* We use abs in all function to get positive number...
	   However, its only necessary in WEEK_DELDAY - now->tm_wday
	   where WEEK_DELDAY can be 0 or 6. */
	if (currentUser->checktype==MONTHUP) { /* Monthly */
		/* Check for period users */
		if (currentUser->status==PERIOD) daysleft = abs(30 - dayspassed);
		else daysleft = abs(d_mon[now->tm_mon] - now->tm_mday);
	} else if (currentUser->checktype==WKUP) { /* Weekly */
		/* Check for period users */
		if (currentUser->status==PERIOD) daysleft = abs(7 - dayspassed);
		else daysleft = abs(WEEK_DELDAY - now->tm_wday);
	} else daysleft=0;

	if (currentUser->groups[0] == '\0') 
		group = nogroup;
	else if (strchr(currentUser->groups,' ') != NULL) 
		group = strsplit_s(tstr,currentUser->groups,' ',0);
	else
		group=currentUser->groups;

	minquota=currentUser->minmb;
	needs=(currentUser->minmb - currentUser->upload);
	uploaded=(currentUser->upload - currentUser->minmb);


	if (currentUser->uClass==TRIAL) {
		if (currentUser->status==PASSED) printf(BOT_TRAIL_PASSED);
		else if (currentUser->status==PERIOD) printf(BOT_TRAIL_PERIOD);
		else printf(BOT_TRAIL_FAILED);
	}
	else if (currentUser->uClass==NORMAL) {
		if (currentUser->status==PASSED) printf(BOT_NORMAL_PASSED);
		else if (currentUser->status==PERIOD) printf(BOT_NORMAL_PERIOD);
		else printf(BOT_NORMAL_FAILED);
	}
	else if (currentUser->uClass==EXEMPT) {
		printf(BOT_EXEMPT);
	}
	else if (currentUser->uClass==DELETED) {
		printf(BOT_DELETED);
	}
}

/**********************************************************
*  Function name: group_botprint()
*  Arguments: (string) group - group name.
*  Returns: None
*  Description:
*    The function will output to stdout multiple lines,
*    defined on jtconf.h according to the group's users status,
*    these lines should be used in conjunction with the tcl
*    script (passedbot.tcl) for an eggdrop (see README).
*  Modified: 30/9/02
*************************************************************/

void group_botprint(const char *group) {
	scrUser *currentUser;
	int grp_found = 0;

	if (IsUserInGroup(group,sExemptGroups)) {
		printf(BOT_EXEMPT_GROUP);
		return;
	}

	for (currentUser = userPrime; currentUser->next != NULL; currentUser = currentUser->next) {
		if (IsUserInGroup(group, currentUser->groups)) {
			grp_found = 1;
			botprint(currentUser->name);
			printf("\n");
		}
	}
	if (!grp_found)
		printf(BOT_GROUP_NOTFOUND);
	
}

/**********************************************************
*  Function name: g_botprint()
*  Arguments: (string) group - group name.
*  Returns: None
*  Description:
*    The function will output to stdout single line of
*    group info like in botprint()
*  Modified: 2/10/02
*************************************************************/

#ifdef GROUP_TRIAL_FILE		
void g_botprint(char *group) {
	int i, daysleft;
	long uploaded=0, minquota=0, needs=0;
	char theperiod[7];
	time_t temp_t;


	temp_t = time(NULL);
	now = localtime(&temp_t);
	for (i=0; i<nTG; i++) {
		if (strcmp(grpArray[i].name, group)==0) {
			if (grpArray[i].uploaded != NULL)
				uploaded = *grpArray[i].uploaded;
			minquota = grpArray[i].minmb;
			needs = grpArray[i].minmb - *grpArray[i].uploaded;
			#ifdef DEBUG
				printf("checktype = %d, tm_mday=%d tm_wday=%d (%d)\n",
					grpArray[i].checktype, now->tm_mday, now->tm_wday,
					abs(d_mon[now->tm_mon] - now->tm_mday));
			#endif
			if (grpArray[i].checktype==MONTHUP) {		/* Monthly */
				daysleft = abs(d_mon[now->tm_mon] - now->tm_mday);
				strcpy(theperiod,"Month");
			} else if (grpArray[i].checktype==WKUP) {	/* Weekly */
				daysleft = abs(WEEK_DELDAY - now->tm_wday);
				strcpy(theperiod,"Week");
			} else {
				daysleft=0;
				strcpy(theperiod,"Day");
			}
			if (grpArray[i].status == PASSED)
				printf(BOT_GT_PASSED);
			else
				printf(BOT_GT_FAILED);
			return;
		}
	}
	printf(BOT_EXEMPT_GROUP);
	
}
#endif
/**********************************************************
*  Function name: printusage()
*  Arguments: (string) progpath - (argv[0]) the path to jtc
*  Returns: None
*  Description: 
*    The function will output to stdout the allowed switches
*    and options with details.
*  Modified: 2/10/02
*************************************************************/

void printusage(char *progpath) {
	printf("*** JeTrial-C v%s by [Jedi] ****\n",VERSION);
	printf("Usage: %s <-d|-n|-f> [-l|-sl|-u|-us|-g|-gs|-pu] [-class|-status]\n", progpath);     
	printf("\
You must select one of the following modes:\n\
  -d\t\t Delete mode, will delete failed user only if time is right (=end of day/week/month)\n\
  -n\t\t Dont delete mode, will not delete users in any case (overrides -f and -d).\n\
  -f\t\t Force delete users mode, even if its not delete time, failed users will be deleted (CAUTION!).\n\
Optional output:\n\
  -l\t\t list all users with their info (long list).\n\
  -sl\t\t list only the names of users (short list, can be used with bot).\n\
  -u <user>\t Show user`s info.\n\
  -us <user>\t Use in conjunction with bot, show user status.\n\
  -g <group>\t Use in conjunction with bot, show all group's users status (multiple lines).\n\
  -gs <group>\t Use in conjunction with bot, show group trial status (if compiled).\n\
  -pu <user> <group>\t Untrial - Put <user> in <group> if user is trial and group isnt exempt.\n\
Optional filter settings:\n\
  -class <normal|trial|exempt>     Filter users by class.\n\
  -status <passed|failed|period>   Filter users by status.\n\
*period - users who have not passed their trial period\n");

}

/*** EOF ***/

