/******************************************************************************
* Filename: jtc.c
* Description: Main source of JeTrial-C
* Author: [Jedi] (jedi_il@hotmail.com)
* Copyright 2002 [Jedi]
* See COPYING for the GPL GNU
*
*    This file is part of JeTrial-C.
*
*    JeTrial-C is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    JeTrial-C is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Foobar; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/


#include <stdio.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/param.h>

#include "../header/strlib.h"
#include "../config/jtconf.h"
#include "../header/proto.h"
#include "../header/extern.h"

char exempt_flags[]=EXEMPT_FLAGS;

int main(int argc, char *argv[]) {
	DIR *gldir;
	FILE *gl_log;
	dirent *gldirent;	
	time_t temp_t;
	char *g_user=NULL, *g_group=NULL;
	int i, g_listtype=0, g_filter_class=0, g_filter_status=0, 
	       g_showuser=0, g_userstats=0, g_groupstats=0, g_putuserin=0;
	struct stat tstat;
	scrUser *currentUser;

	#ifdef GROUP_TRIAL_FILE
		FILE *f_grpfile;
		char *buff;
		int g_gtstats=0;
	#endif

	atexit(terminated);

	if ( (userPrime=list_init()) ==NULL) exiterror(MALLOCERROR);	/* initilize the list */
	temp_t=time(NULL);
	now=localtime(&temp_t);
	
		
	if (stat(PASSWDFILE,&tstat)==-1) {
		printf("Error %d: %s (%s)\n",errno,strerror(errno),PASSWDFILE);
		exit(-1);
	}
	if (stat(GROUPFILE,&tstat)==-1) {
		printf("Error %d: %s (%s)\n",errno,strerror(errno),PASSWDFILE);
		exit(-1);
	}
	if (stat(GLUSERDIR,&tstat)==-1) {
		printf("Error %d: %s (%s)\n",errno,strerror(errno),GLUSERDIR);
		exit(-1);
	}
	
	if (argc==1) { printusage(argv[0]); exit(0); }
	/* Get exempt groups list */
	getexemptgroups();
	
	for (i=1;i<argc;i++) {
		if (strcmp(argv[i],"-n")==0) g_delmode=DONTDEL;
		else if ((strcmp(argv[i],"-d")==0) && (g_delmode!=DONTDEL) && (g_delmode!=FORCEDEL)) 
			g_delmode=NORMALDEL;
		else if ((strcmp(argv[i],"-f")==0) && (g_delmode!=DONTDEL)) g_delmode=FORCEDEL;
		else if (strcmp(argv[i],"-l")==0) g_listtype=LONGLIST;
		else if (strcmp(argv[i],"-sl")==0) g_listtype=SHORTLIST;
		else if (strcmp(argv[i],"-class")==0) {
			if ((i+1)>=argc) {
				printf("Usage: %s -class <normal|trial|exempt>\n",argv[0]);
				exit(0);
			}
			if (strcmp(argv[i+1],"normal")==0) g_filter_class=NORMAL;
			else if (strcmp(argv[i+1],"trial")==0) g_filter_class=TRIAL;
			else if (strcmp(argv[i+1],"exempt")==0) g_filter_class=EXEMPT;
			else {
				printf("Usage: %s -class <normal|trial|exempt>\n",argv[0]);
				exit(0);
			}
		}
		else if (strcmp(argv[i],"-status")==0) {
			if ((i+1)>=argc) {
				printf("Usage: %s -status <passed|failed|period>\n",argv[0]);
				exit(0);
			}
			if (strcmp(argv[i+1],"passed")==0) g_filter_status=PASSED;
			else if (strcmp(argv[i+1],"failed")==0) g_filter_status=FAILED;
			else if (strcmp(argv[i+1],"period")==0) g_filter_status=PERIOD;
			else {
				printf("Usage: %s -status <passed|failed|period>\n",argv[0]);
				exit(0);
			}
		}
		else if (strcmp(argv[i],"-u")==0) { 
			if ((i+1)>=argc) { 
				printf("Usage: %s -u <username>\n",argv[0]);
				exit(0);
			}
			g_showuser=1; 
			g_user=argv[i+1];
		}
		else if (strcmp(argv[i],"-us")==0) {
			if ((i+1)>=argc) {
				printf("Usage: %s -us <username>\n",argv[0]);
				exit(0);
			}
			g_userstats=1;
			g_user=argv[i+1];
		}
		else if (strcmp(argv[i],"-pu")==0) {
			if ((i+2)>=argc) {
				printf("Usage %s -pu <user> <group>\n",argv[0]);
				exit(0);
			}
			g_putuserin=1;
			g_user=argv[i+1];
			g_group=argv[i+2];
		}
		else if (strcmp(argv[i],"-g")==0) {
			if ((i+1)>=argc) {
				printf("Usage: %s -gs <groupname>\n",argv[0]);
				exit(0);
			}
			g_groupstats=1;
			g_group=argv[i+1];
		}
		else if (strcmp(argv[i],"-gs")==0) {
			#ifdef GROUP_TRIAL_FILE
			if ((i+1)>=argc) {
				printf("Usage: %s -gt <groupname>\n",argv[0]);
				exit(0);
			}
			g_gtstats=1;
			g_group=argv[i+1];
			#else
			printf("This function is unavailable since it wasnt compiled in.\n");
			exit(0);
			#endif
		}
		
	}
	/* If no del mode is chosen */
	if (g_delmode==0) { 
		printusage(argv[0]); 
		printf("\nPlease select one of the three delete mode (del/force/dont)\n");
		exit(0); 
	}

	/* Collect groups data... */
	#ifdef GROUP_TRIAL_FILE
                f_grpfile = fopen(GROUP_TRIAL_FILE,"r");
                if (f_grpfile != NULL) {
                        buff = (char *)malloc(256);
			nTG=0;
                        while (fgets(buff,256,f_grpfile)!=NULL) if (strchr(buff,' ')) nTG++;
                        grpArray = (scrGroup *)malloc(nTG * sizeof(scrGroup));
                        rewind(f_grpfile);
                        for (i=0; i<nTG; i++) {
				if (fgets(buff,256,f_grpfile)==NULL) break;
                                strncpy(grpArray[i].name, strsplit_s(tstr,buff,' ',0), 24);
				grpArray[i].minmb = atol( strsplit_s(tstr,buff,' ',1) );
				if ( strncmp(strsplit_s(tstr,buff,' ',2), "MONTHUP", 7) == 0 ) {
					grpArray[i].uploaded = &grpArray[i].monthup;
					grpArray[i].checktype = MONTHUP;
				} else if ( strncmp(tstr, "WKUP", 4) == 0 ) {
					grpArray[i].uploaded = &grpArray[i].wkup;
					grpArray[i].checktype = WKUP;
				} else if ( strncmp(tstr, "DAYUP", 5) == 0 ) {
					grpArray[i].uploaded = &grpArray[i].dayup;
					grpArray[i].checktype = DAYUP;
				} else {
					grpArray[i].uploaded = &grpArray[i].allup;
					grpArray[i].checktype = NA;
				}
                        }
                        fclose(f_grpfile);
                        free(buff);
                }
	#endif

	/* Collecting data... */
	gldir=opendir(GLUSERDIR);
	while ((gldirent=readdir(gldir))) { 		/* Lists all files in users dir */
		if (
			(strncmp(gldirent->d_name,".",1)!=0) &&
			(strncmp(gldirent->d_name,"..",1)!=0) &&
			(strncmp(gldirent->d_name,"default.",8)!=0)
		)	readuser(gldirent->d_name);
	}
	closedir(gldir);

	/* Users have been read, Check if groups have failed/passed */
	#ifdef GROUP_TRIAL_FILE
	   for (i=0; i<nTG; i++) {
		if ( grpArray[i].uploaded != NULL && 
		     *grpArray[i].uploaded >= grpArray[i].minmb)
			grpArray[i].status = PASSED;
		else 	grpArray[i].status = FAILED;
		if ( (isDelTime(grpArray[i].checktype)==1 || g_delmode==FORCEDEL) && g_delmode!=DONTDEL )
			delgroup(grpArray+i);
	   #ifdef DEBUG
		printf("Group(%-8s) MONTHUP(%ld) WKUP(%ld) DAYUP(%ld) ALLUP(%ld) PASSED(%d)\n",
			grpArray[i].name, grpArray[i].monthup, grpArray[i].wkup, grpArray[i].dayup,
			grpArray[i].allup, grpArray[i].status);
	   #endif
	   }
	#endif
	
	if (g_listtype>0) printlist(g_filter_class,g_filter_status,g_listtype);	

	if (g_showuser) printuser(g_user);
	else if (g_userstats) botprint(g_user);
	else if (g_groupstats) group_botprint(g_group);
       #ifdef GROUP_TRIAL_FILE
	else if (g_gtstats) g_botprint(g_group);
       #endif
	else if (g_putuserin) {
		if (IsUserInGroup(g_group,sExemptGroups)) {
			printf(UNTRIAL_EXEMPT);
			exit(0);
		}
		if ((currentUser=find_by_name(g_user,userPrime))==NULL) {
			printf(UNTRIAL_NOTONSITE);
			exit(0);
		}
		if (currentUser->uClass!=TRIAL) {
			printf(UNTRIAL_NONTRIAL);
			exit(0);
		}
		if (currentUser->status!=PASSED) {
			printf(UNTRIAL_NOTPASSED);
			exit(0);
		}
		putuserin(g_user,g_group);
		printf(UNTRIAL_SUCCESS);
	}

	if (stats.total_deleted > 0) { /* log to glftpd.log */
		gl_log=fopen(GLFTPDLOG,"a");
		if (gl_log != NULL) {
			if (strlen(dy_deleted)>0)
				fprintf(gl_log,"DELUSERS: \"DAY\" \"%s\"",dy_deleted);
			if (strlen(wk_deleted)>0) 
				fprintf(gl_log,"DELUSERS: \"WEEK\" \"%s\"",wk_deleted);
			if (strlen(mn_deleted)>0)
				fprintf(gl_log,"DELUSERS: \"MONTH\" \"%s\"",mn_deleted);
		     #ifdef GROUP_TRIAL_FILE
			if (strlen(grp_deleted)>0)
				fprintf(gl_log,"DELGROUPS: \"%s\"", grp_deleted);
		     #endif
			fclose(gl_log);
		}
	}
	return 0;
}

void terminated(void) {
	#ifdef DEBUG
	printf("Freeing list...\n");
	#endif

	listfree(userPrime);
	if (sExemptGroups) free(sExemptGroups);

	#ifdef DEBUG
	printf("Done, end of prog\n");
	#endif
}

/**********************************************************
*  Function name: getexemptgroups()
*  Arguments: None.
*  Returns: None.
*  Description:
*    The function will process the sExemptGroups list according
*    the precompiled directives EXEMPT_GROUPS_DIRS and 
*    EXEMPT_GROUPS_FILE.
*  Modified: 10/6/02
*************************************************************/

void getexemptgroups(void) {
#ifdef EXEMPT_GROUPS_FILE
	FILE *f_egf;
	char buffer[40];
#endif
#ifdef EXEMPT_GROUPS_DIRS
	struct dirent *dptr;
	DIR *dstream;
	char egrp_dirs[] = EXEMPT_GROUPS_DIRS, *ptr;
#endif
	sExemptGroups = (char *)malloc(sizeof(EXEMPT_GROUPS)+1);
	if (sExemptGroups == NULL) exiterror(MALLOCERROR);
	strcpy(sExemptGroups, EXEMPT_GROUPS);
#ifdef EXEMPT_GROUPS_FILE
	/* Read exempt groups file */
	f_egf = fopen(EXEMPT_GROUPS_FILE,"r");
	if (f_egf != NULL) {
		while (fgets(buffer,sizeof(buffer),f_egf)) {
			if (strlen(buffer) < 2) continue;
			buffer[strlen(buffer)-1] = '\0'; /* remove newline */
			sExemptGroups = (char *)realloc(sExemptGroups,strlen(sExemptGroups) + strlen(buffer)+1);
			sprintf(sExemptGroups, "%s %s", sExemptGroups, buffer);
		}
		fclose(f_egf);
	}
#endif
#ifdef EXEMPT_GROUPS_DIRS
	/* Read exempt directories */
	for (ptr = egrp_dirs; (ptr = strtok(ptr," ")) != NULL ; ptr = NULL) {
		dstream = opendir(ptr);
		if (dstream == NULL) continue;
		while ((dptr = readdir(dstream)) != NULL) {
			if (strcmp(dptr->d_name,".")==0 || strcmp(dptr->d_name,"..")==0) continue;
			sExemptGroups = (char *)realloc(sExemptGroups, strlen(sExemptGroups) + strlen(dptr->d_name)+1);
			sprintf(sExemptGroups, "%s %s", sExemptGroups, dptr->d_name);
		}
		closedir(dstream);
	}
#endif
#ifdef DEBUG
	printf("Exempted groups = '%s'\n",sExemptGroups);
#endif
}

/**********************************************************
*  Function name: readuser()
*  Arguments: (string) uUserName
*  Returns: None.
*  Description:
*    The function will get initial data from disk, hence
*    will process users dir and passwd file, will create
*    a linekd list of all users, will fill wkup, monthup, dayup,
*    groups, flags, addedon.
*    The function  will also skip non-user or deleted users files.
*  Modified: 30/9/02
*************************************************************/

void readuser(const char *sUserName) {
	scrUser *currentUser = NULL;
	FILE *fUser,*fPasswd;
	char sLine[LINELEN], buff[256], sdate[20];
	struct tm *T;
	int isUser=0;
	#ifdef GROUP_TRIAL_FILE
		int i;
	#endif

	sprintf(buff,"%s%s",GLUSERDIR,sUserName);	/* Set path GLUSERDIR\username */
	fUser=fopen(buff,"r");			/* Open file for reading */
	if (fUser==NULL) {
		printf("Error %d: %s (%s) userfile\n",errno,strerror(errno),buff);
		return;
	}

	/* Allocate memory for new item */
	currentUser = alloc_item();
	if (currentUser == NULL) exiterror(MALLOCERROR);

	/* Set the username */
	if (strlen(sUserName) < sizeof(currentUser->name) )
		strcpy(currentUser->name,sUserName);
	else return;

	#ifdef DEBUG
		printf("----------------[ Processing %s ]----------------\n",sUserName);
	#endif
	while (fgets(sLine,LINELEN,fUser)!=NULL) {
		chomp(sLine);
		if (strncmp(sLine,"FLAGS",5)==0) {		/* This line is FLAGS line */
			isUser=1;
			if (strlen(sLine+6) < sizeof(currentUser->flags) )
				strcpy(currentUser->flags, sLine+6);
			#ifdef DEBUG
			else	printf("Could not set flags for user %s - line too long!\n", currentUser->name);
			#endif
			#ifdef DEBUG
				printf("FLAGS: %s\n",currentUser->flags);
			#endif
		}
		else if (strncmp(sLine,"GROUP",5)==0) {		/* GROUP */

			if (strlen(currentUser->groups) + strlen(sLine+6) < sizeof(currentUser->groups) ) {
				if (currentUser->groups[0] == '\0') {
					strcpy(currentUser->groups, sLine+6);
				} else {
					sprintf(currentUser->groups,"%s %s",currentUser->groups, sLine+6);
				}
			}
			#ifdef DEBUG
			else	printf("Could not add group %s to %s, grouplist reached max of %d chars !",
					sLine+6, currentUser->name, sizeof(currentUser->groups) );

				printf("GROUPS: %s\n",currentUser->groups);
			#endif
		}
		else if (strncmp(sLine,"ALLUP",5)==0) {		/* ALL UP */
			currentUser->allup=totalup(sLine);
			#ifdef DEBUG
				printf("ALLUP: %ldMb\n",currentUser->allup);
			#endif
		}
		else if (strncmp(sLine,"WKUP",4)==0) {		/* WEEK UP */
			currentUser->wkup=totalup(sLine);
			#ifdef DEBUG
				printf("WKUP: %ldMb\n",currentUser->wkup);
			#endif
		}
		else if (strncmp(sLine,"MONTHUP",7)==0) {	/* MONTH UP */
			currentUser->monthup=totalup(sLine);
			#ifdef DEBUG
				printf("MOTNUP: %ldMb\n",totalup(sLine));
			#endif
		}
		else if (strncmp(sLine,"DAYUP",5)==0) {		/* DAY UP */
			currentUser->dayup=totalup(sLine);
			#ifdef DEBUG
				printf("DAYUP: %ldMb\n",currentUser->dayup);
			#endif
		}
	}/* while */
	fclose(fUser);
	/* check if fUser is a userfile... */

	if (isUser==0) { /* its not a userfile */
		#ifdef DEBUG
			printf("Not a userfile\n");
		#endif
		free_item(currentUser);
		return;
	} else {
		/* Its a user, lets link it to the list */
		if ( link_item(userPrime, currentUser) ) 
			exiterror("Could not link empty user!");
	}

	fPasswd=fopen(PASSWDFILE,"r");
	if (fPasswd==NULL) { 
		printf("Error %d: %s (%s)\n",errno,strerror(errno),PASSWDFILE);
		return; 
	}
	while (fgets(sLine,LINELEN,fPasswd)!=NULL) {
		if(strncmp(sLine,sUserName,strlen(sUserName))==0) {
			if (strlen(strsplit_s(sdate,sLine,':',4))==8) {
				T=(struct tm *)calloc(1,sizeof(struct tm)); /* allocate memory */
				if (T==NULL) exiterror(MALLOCERROR);
				T->tm_mon=atoi(strsplit_s(tstr,sdate,'-',0))-1; /* month */
				T->tm_mday=atoi(strsplit_s(tstr,sdate,'-',1));  /* day   */
				T->tm_year=atoi(strsplit_s(tstr,sdate,'-',2));  /* year  */
				if (T->tm_year < 50) T->tm_year+=100;
				currentUser->addedon = mktime(T);
				free(T);
			}
		
			#ifdef DEBUG
				printf("AddedOn: %ld\n",currentUser->addedon);
			#endif
			break; /* quit from the loop */
		}
	} /* while */
	fclose(fPasswd);
	#ifdef DEBUG
		if (currentUser->addedon == 0) {
			printf("Warning: could not find user %s in passwd file!\n",currentUser->name);
		}
	#endif

	/* Add user stats to his group */
	#ifdef GROUP_TRIAL_FILE
		for (i=0; i<nTG; i++) {
			if (IsUserInGroup(currentUser->groups, grpArray[i].name)) {
				grpArray[i].allup   += currentUser->allup;
				grpArray[i].monthup += currentUser->monthup;
				grpArray[i].wkup    += currentUser->wkup;
				if (grpArray[i].nou <= MAX_UPG) 
					grpArray[i].users[grpArray[i].nou++] = currentUser;
			}
		}
	#endif
	handleuser(currentUser);	/* Handle user... */
}

/**********************************************************
*  Function name: handleuser()
*  Arguments: pointer to user structure
*  Returns: None
*  Description:
*    The function will proccess the user, will fill the following
*    specs: class, status, minmb, deleted.
*    the function will check the user against groups/flags
*    to set the corret class and status, and if required,
*    will call upon deleting the user.
*  Modified: 23/8/02
*************************************************************/

void handleuser(scrUser *currentUser) {
	int x, deluser_flag=0;
	struct tm *tp;

	/* Check if user is not deleted */
	if (strstr(currentUser->flags,DELETED_FLAG)!='\0') {
		currentUser->uClass=DELETED;
		return;
	}

	/* By default all users are normal, set it up: */
	currentUser->uClass=NORMAL;
	currentUser->minmb=NORMAL_MINMB;
	fillupload(NORMAL_CHECK,currentUser);
	currentUser->deleted=0;

	/* Check if user has groups at all */
	if (currentUser->groups!='\0') {
		if (IsUserInGroup(currentUser->groups,TRIAL_GROUPS)) {
			currentUser->uClass=TRIAL;
			currentUser->minmb=TRIAL_MINMB;
			fillupload(TRIAL_CHECK,currentUser);
		}
		if (IsUserInGroup(currentUser->groups,sExemptGroups)) {
			currentUser->uClass=EXEMPT;
			currentUser->minmb=0;
			currentUser->status=NA;
			fillupload("",currentUser);
		}
	}

	/* check for exempt by flags (updated)*/
	for (x=0; x<sizeof(EXEMPT_FLAGS)-1; x++) {
		if (strchr(currentUser->flags, exempt_flags[x])!=NULL) {
			currentUser->uClass=EXEMPT;
			currentUser->minmb=0;
			currentUser->status=NA;
			fillupload("",currentUser);
		}
	}
	/* check PERIOD users, use ALLUP if user has been less than twice his 
	   minimal testing period.
	   we use twice becuase after 1 period is ended, but the week/month has 
	   not ended, the program would use monthup/wkup and this might cause
	   the user to fail although last month he as uploaded the minimal quota */

	tp = localtime(&currentUser->addedon);
	if (currentUser->checktype == MONTHUP)
		tp->tm_mday=1;
	else if (currentUser->checktype == WKUP)
		tp->tm_wday=0;
	if ((time(NULL) - mktime(tp)) < 2 * getperiod(currentUser)) {
		fillupload("ALLUP",currentUser);
	}

	/* if its not Exempt nor trial, its normal */
	#ifdef DEBUG	
		printf("CLASS: %s (%d)\n",getclass[currentUser->uClass],currentUser->uClass);
		printf("UPLOAD: %ld MINMB: %ld\n",currentUser->upload,currentUser->minmb);
	#endif
	/* Check if user has passed period */
	if (currentUser->uClass!=EXEMPT) {
		if (currentUser->upload >= currentUser->minmb) {
			currentUser->status=PASSED;
			stats.total_passed++;
		}
		else if ((time(NULL)-currentUser->addedon) < getperiod(currentUser) - HALFDAY) {
			/* Period users checked against their trail period minus half a day (43200)
			   this is becuase we want to delete the user on a specific day after his
			   trial period is over, we give half a day window for jtc to run when in this
			   period he will be deleted */
			currentUser->status=PERIOD;
			stats.total_period++;
		}
		else { 
			currentUser->status=FAILED; 
			stats.total_failed++; 
		}
	} /* if */
	
	#ifdef DEBUG	
		printf("STATUS: %s(%d)\n",getstatus[currentUser->status],currentUser->status);
	#endif
	/* Add to count */
	stats.total_users++;
	switch (currentUser->uClass) {
		case EXEMPT: stats.total_exempt++; break;
		case TRIAL : stats.total_trial++; break;
		case NORMAL: stats.total_normal++; break;
	}
	/* if user failed, and now is deltime, delete user */
	if ((currentUser->status==FAILED) && (currentUser->uClass!=EXEMPT)) { 
	#ifdef ENABLE_WARNING_MSG
		if (now->tm_mday==(d_mon[now->tm_mon]-MSG_DAYS)) { /* Its MSG_DAYS before MONTH end (del) */
			if (currentUser->checktype==MONTHUP)
				sendmsg(currentUser);
		}
	#endif

		deluser_flag = isDelTime(currentUser->checktype);
		if ((time(NULL)-currentUser->addedon) < getperiod(currentUser)) {
			/* User (period) is in range, defined failed, his trial has ended */
			deluser_flag=1;
		}

		if ( (deluser_flag || g_delmode==FORCEDEL) && g_delmode!=DONTDEL )
			deluser(currentUser);
		/* if the never delete flag is not flagged (!g_dontdel), AND either
		   deluser_flag OR g_forcedel (-f option) are flagged, only then delete user. */
	} /* if */
	
}

/**********************************************************   
*  Function name: isDelTime()
*  Arguments: (int) checktype (WKUP/DAYUP/MONTHUP)
*  Returns: 1 if its deltime, 0 if it isnt
*  Description:
*    Deltime is a specific time in the day/week/month where we
*    delete users, it happens just before reset.
*  Modified: 10/1/02
*************************************************************/

int isDelTime(int checktype) {
	if (now->tm_mday==d_mon[now->tm_mon] && checktype==MONTHUP) return 1; 	/* Its MONTH del */
	else if (now->tm_wday==WEEK_DELDAY && checktype==WKUP) return 1; 	/* Its WEEK del */
	else if (now->tm_hour==DAY_DELHOUR && checktype==DAYUP) return 1;	/* Its DAY del */
	else return 0;
}

/**********************************************************
*  Function name: totalup()
*  Arguments: (string) line from userfile of WKUP/MONTHUP/DAYUP
*  Returns: a long int of the total sum of all sections
*           which the user uploaded, in megabytes.
*  Description: 
*    The function will read the line and will sum up all
*    uploaded kb, from all sections, will return the sum in Mb.
*  Modified: 14/7/02
*************************************************************/

long totalup(const char *line) {
	char *ch;
	int x;
	long ul=0,v;
	ch=(char *)malloc(strlen(line)+1);
	for (x=2; *strsplit_s(ch,line,' ',x) != '\0' ; x+=3) {
		v=atol(ch);
		ul+=(long)(v/1024);
	}
	free(ch);
	return ul; 
}

/**********************************************************
*  Function name: getperiod()
*  Arguments: pointer to user struct - currentUser
*  Returns: long int representing caladner time.
*  Description:
*    The function counts the amount of minimal time which 
*    had to pass since user had been added, to be accounted as
*    a tested user -passed or failed- (aka, not having PERIOD status)
*    accoding to its checktype (WKUP, MONTHUP, DAYUP).
*  Modified: 14/7/02
*************************************************************/

long getperiod(scrUser *currentUser) {
	if (currentUser->checktype==WKUP) return WEEKSEC;
	else if (currentUser->checktype==MONTHUP)  return MONTHSEC;
	else if (currentUser->checktype==DAYUP) return DAYSEC;
	else return 0;
}

/**********************************************************
*  Function name: fillupload()
*  Arguments: checktype - "WKUP", "DAYUP", "MONTHUP"
*	      currentUser - pointer to user struct
*  Returns: None
*  Description: 
*    The function will fill currentUser's 'upload' and 'checktype'
*    arguments, according to the received checktype.
*  Modified: 14/7/02
*************************************************************/

void fillupload(const char *checktype, scrUser *currentUser) {
	if (strcmp("WKUP",checktype)==0) {
		currentUser->upload=currentUser->wkup;
		currentUser->checktype=WKUP;
	}
	else if(strcmp("MONTHUP",checktype)==0) {
		currentUser->upload=currentUser->monthup;
		currentUser->checktype=MONTHUP;
	}
	else if(strcmp("DAYUP",checktype)==0) {
		currentUser->upload=currentUser->dayup;	
		currentUser->checktype=DAYUP;
	}
	else if(strcmp("ALLUP",checktype)==0) {
		currentUser->upload=currentUser->allup;
		/* We do not set checktype, becuase periodic users
		   use normal/trial users checktype */
	}
else {
		currentUser->upload=0;
		currentUser->checktype=NA;
	}
}

/**********************************************************
*  Function name: deluser()
*  Arguments: pointer to struct scrUser.
*  Returns: None
*  Description: 
*    The function will "delete" the currentUser, 
*    What it will actually do is will add deleted flag (6)
*    to the user, and mark him as deleted. it will also log the action.
*  Modified: 1/8/02
*************************************************************/

void deluser(scrUser *currentUser) {

	int fd;
	char path[MAXPATHLEN], path2[MAXPATHLEN], sLine[LINELEN], *ptrdel=NULL;
	FILE *fUser, *tmpfile;
#ifdef ENABLE_BYE_FILE
	char byemsg[MSG_LEN];
	long uploaded, required;
#endif
#ifdef DELUSER_LOG
	FILE *deluserlog;
	char buff[256], *timestr;
	time_t temp_t;
#endif

#ifdef ENABLE_BYE_FILE
	uploaded=currentUser->upload;
	required=currentUser->minmb;
	sprintf(path,"%s%s.bye",BYEFILES_PATH,currentUser->name);
	sprintf(byemsg,BYE_MSG);
	tmpfile=fopen(path,"a+");
	if (tmpfile==NULL) {
		printf("Error %d: %s (%s) bye file\n",errno,strerror(errno),path);
	} else {
		fputs(byemsg,tmpfile);
		fclose(tmpfile);
	}
#endif

	sprintf(path,"%s%s",GLUSERDIR,currentUser->name);
	sprintf(path2,"%s.XXXXXX",path);	/* temp file's path */
	fd = mkstemp(path2);	/* Create temp file to write to */
	if( fd == -1 ) { 
		printf("Cant open '%s' for writing!\n",path2);
		return;
	}
	tmpfile = fdopen(fd,"w");
	if (tmpfile==NULL) {
		printf("Error %d: %s (%s)\n",errno,strerror(errno),PASSWDFILE);
		return;
	}
	fUser = fopen(path,"r");
	if (fUser==NULL) {
		printf("Error %d: %s (%s)\n",errno,strerror(errno),path);
		fclose(tmpfile);
		return;
	}
	while (fgets(sLine,LINELEN,fUser) != '\0') {
		if (strncmp(sLine,"FLAGS",5)==0) {
			sLine[strlen(sLine)-1]='\0'; /* remove \n    */
			strcat(sLine,"6\n");	/* adds 6\n          */
		} 
		fprintf(tmpfile,"%s",sLine);
	} /* while */
	fclose(fUser);
	fclose(tmpfile);
	rename(path2,path); /* move temp file over orig file */


#ifdef DELUSER_LOG
	temp_t=time(NULL);
	chomp(timestr=ctime(&temp_t));
	sprintf(buff,"%s Deleted %-14s (%-14s) uploaded: %5ldMb required: %5ldMb class: %-5s\n",
		timestr, currentUser->name, currentUser->groups, currentUser->upload,
		currentUser->minmb,getclass[currentUser->uClass]);
	#ifdef DEBUG
		printf("Logging: %s\n",buff);
	#endif
	deluserlog=fopen(DELUSER_LOG,"a+");
	if (deluserlog==NULL) {
		printf("Error %d: %s (%s) DELUSER_LOG file\n",errno,strerror(errno),DELUSER_LOG);
	} else {
		fputs(buff,deluserlog);
		fclose(deluserlog);
	}
#endif
	currentUser->deleted=1;
	stats.total_deleted++;
	switch (currentUser->checktype) {
		case DAYUP:   ptrdel=dy_deleted; break;
		case WKUP:    ptrdel=wk_deleted; break;
		case MONTHUP: ptrdel=mn_deleted; break;
	}
	if (ptrdel != NULL) {
		if (strlen(ptrdel) < DELSIZE-25) {
			if (*ptrdel != '\0') strcat(ptrdel," ");
			strcat(ptrdel,currentUser->name);
		}
	}
}

/**********************************************************
*  Function name: delgroup()
*  Arguments: pointer to struct scrGroup.
*  Returns: None
*  Description:
*    The function will "delete" the current group
*  Modified: 4/10/02
*************************************************************/

#ifdef GROUP_TRIAL_FILE
void delgroup(scrGroup *delgrp) {
	int i;
	scrUser *currentUser;

	delgrp->deleted = 1;
	for (i=0; i < delgrp->nou; i++) {
		currentUser = delgrp->users[i];
		/* Check if user is not an exempt user */
		if (currentUser->uClass != EXEMPT)		
			deluser(currentUser);
	}

}
#endif



/**********************************************************
*  Function name: exiterror()
*  Arguments: (string) strerror - a description of error
*  Returns: None
*  Description:
*    The function will quit the prog with strerror printed
*    to stderr.
*  Modified: 16/7/02
*************************************************************/

void exiterror(const char *strerror) {
	fprintf(stderr,"%s\n",strerror);
	exit(0);
}

/**********************************************************
*  Function name: writemsg()
*  Arguments: currentUser
*  Returns: None
*  Description:
*    The function will send a msg to a user which havent
*    reached minimal quota at specific date.
*  Modified: 31/7/02
*************************************************************/
#ifdef ENABLE_WARNING_MSG
void sendmsg(scrUser *currentUser) {
	FILE *fn;
	char str[MAXPATHLEN];
	sprintf(str,"%s%s",MSG_PATH,currentUser->name);
	fn=fopen(str,"a+");
	if (fn==NULL) {
		printf("Error %d: %s (%s) warning msg\n",errno,strerror(errno),PASSWDFILE);
		return;
	}
	fputs(WARNING_MSG,fn);
	fclose(fn);
}
#endif

/**********************************************************
*  Function name: putuserin()
*  Arguments: (string) username, (string) groupname
*  Returns: None
*  Description:
*    The function will put username in groupname, and will
*    remove username from any trial groups.
*  Modified: 2/8/02
*************************************************************/

void putuserin(char *username, char *groupname) {
	scrUser *currentUser = NULL;
	FILE *fGrp, *fUser, *tmpfile;
	int grpexists=0, fd, newgrp_written=0;
	long gid=0,newgid=0;
	char sLine[LINELEN], path[MAXPATHLEN],path2[MAXPATHLEN];

	if (IsUserInGroup(groupname, sExemptGroups)) return; /* We dont want trial users be added to exempt grps. */

	fGrp=fopen(GROUPFILE,"r");
	if (fGrp==NULL) {
		printf("Error %d: %s (%s)\n",errno,strerror(errno),GROUPFILE);
		return;
	}
	while (fgets(sLine,LINELEN,fGrp)!=NULL) {
		if (strncmp(strsplit_s(tstr,sLine,':',0),groupname,strlen(groupname))==0) {
			grpexists=1;
			break;
		} /* Ok, group exists, add user to group. */

		gid=atol(strsplit_s(tstr,sLine,':',2));
		if (gid>=newgid) newgid=gid;
	}
	fclose(fGrp);
	if (!grpexists) {
		/* Create group */
		fGrp=fopen(GROUPFILE,"a+");
		if (fGrp==NULL) {
			printf("Error %d: %s (%s) cant add group, untrial failed!\n",errno,strerror(errno),PASSWDFILE);
			return;
		}
		fprintf(fGrp,"%s::%ld:\n",groupname,newgid+100);
		fclose(fGrp);
	}

	/* Once group fully exists, add user to that group */
	sprintf(path,"%s%s",GLUSERDIR,currentUser->name);
	sprintf(path2,"%s.XXXXXX",path);	/* temp file's path */
	fd = mkstemp(path2);	/* Create temp file to write to */
	if( fd == -1 ) { 
		printf("Cant open '%s' for writing!\n",path2);
		return;
	}
	tmpfile = fdopen(fd,"w");
	if (tmpfile==NULL) {
		printf("Error %d: %s (%s)\n",errno,strerror(errno),path2);
		return;
	}
	fUser = fopen(path,"r");
	if (fUser==NULL) {
		printf("Error %d: %s (%s)\n",errno,strerror(errno),path);
		fclose(tmpfile);
		return;
	}
	while (fgets(sLine,LINELEN,fUser) != '\0') {
		if (strncmp(sLine,"GROUP",5)==0) {
			sLine[strlen(sLine)-1]='\0'; /* remove \n      */
			if ((IsUserInGroup(sLine+6,TRIAL_GROUPS)) && (!newgrp_written)) {
				/* Ok dont write it */
				fprintf(tmpfile,"GROUP %s\n",groupname);
				newgrp_written=1;
			} else {
				fprintf(tmpfile,"%s\n",sLine);
			}
		} else fprintf(tmpfile,"%s",sLine);
	}/* while */
	fclose(fUser);
	fclose(tmpfile);
	rename(path2,path); /* move temp file over orig file */
}

/**********************************************************
*  Function name: IsUserInGroup()
*  Arguments: (string) usergroup, (string) groups
*  Returns: (1) true (0) flase
*  Description:
*    The function will check if usergroup(s) is in one/more
*    of groups, return 1 if so, 0 otherwise. (supports * wildcard)
*  Modified: 2/8/02
*************************************************************/

int IsUserInGroup(const char *usergroup, const char *groups) {
	char *nc_usergroup, *nc_groups, stest[LINELEN]={0}, grp[LINELEN]={0};
	int x;

	/* if groups is nada return 0 */
	if (*groups == '\0') return 0;

	/* Case insentivity */
	nc_groups = strtolower(groups); /* "groupa groupb groupc" */
	nc_usergroup = (char *)malloc(strlen(usergroup)+3);

	sprintf(nc_usergroup," %s ",strtolower(usergroup)); /* " groupa groupb groupc " */
	if ((*strsplit_s(tstr,nc_groups,' ',0))=='\0') x=-1;	/* negative value to strsplit returns the string itself */
	else x=0;	/* While string without separator (' ' spaces)=when only one group is used in groups, returns null. */
	while ((*strsplit_s(stest,nc_groups,' ',x++)) != '\0') { /* stest="groupa" */
		if ((stest[0]=='*') && (stest[strlen(stest)-1]=='*')) { /* if stest=="*...*" */
			stest[strlen(stest)-1]='\0';  /* remove the airstrike(*)         */
			strcpy(grp,stest+1);    /* dont indluce the first airstrike(*)             */
		}
		else if (*stest=='*') { /* if stest=="*..." */
			sprintf(grp,"%s ",stest+1);     /* grp="..."  */
		}
		else if (stest[strlen(stest)-1]=='*') { /* if stest=="...*"          */
			stest[strlen(stest)-1]='\0';  /* remove the last airstrike(*)    */
			sprintf(grp," %s",stest);       /* grp=" groupa" */
		}
		else { /* if stest="..." */
			sprintf(grp," %s ",stest); /* grp=" groupa " */
		}
		/* Ok, now grp is ready for strstr */
		if (strstr(nc_usergroup,grp)!=NULL) {/* User is in the group */
			free(nc_groups);
			free(nc_usergroup);
			return 1;       /* return true */
		}
	} /* while */
	free(nc_groups);
	free(nc_usergroup);
	return 0; /* if no group was found return false */
}

/*** EOF ***/

