/******************************************************************************
* Filename: linked.c
* Description: linked list structure maintance source code
* Author: [Jedi] (jedi_il@hotmail.com)
* Copyright 2002 [Jedi] 
* See COPYING for the GPL GNU
*
*    This file is part of JeTrial-C.
*
*    JeTrial-C is free software; you can redistribute it and/or modify      
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    JeTrial-C is distributed in the hope that it will be useful,         
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Foobar; if not, write to the Free Software 
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "../header/proto.h"

scrUser *list_init(void) {	// returns anchor prime
	scrUser *l;
	l=(scrUser *)calloc(1,sizeof(struct scrUser));
	if (l==NULL) return NULL;	// calloc failed, return null
	l->next=NULL;
	return l;
}

int free_item(scrUser *item) {
	if (item == NULL) return 1;
	//if (item->flags != NULL) free(item->flags);
	//if (item->groups != NULL) free(item->groups);
	free(item);
	return 0;
}

scrUser *alloc_item(void) {
	scrUser *item;
	item = (scrUser *)calloc(1,sizeof(struct scrUser));
	if (item == NULL) return NULL;
	item->next = NULL;
	memset(item->name,   '\0', sizeof(item->name)   );
	memset(item->flags,  '\0', sizeof(item->flags)  );
	memset(item->groups, '\0', sizeof(item->groups) );

	//item->flags = NULL;
	//item->groups = NULL;
	return item;
}

int link_item(scrUser *anchor, scrUser *item) {
	scrUser *last;
	if (item==NULL) return 1;
	last = last_entry(anchor);
	last->next = item;
	return 0;
}

/***************************************** Unused functions
scrUser *new_item(scrUser *anchor) {
	scrUser *prev,*new;
	prev=last_entry(anchor);
	new=(scrUser *)calloc(1,sizeof(struct scrUser));
	if (new==NULL) return NULL;	// calloc failed, return null
	prev->next=new;
	new->next=NULL;
	return new;
}

void del_item(scrUser *delme, const scrUser *anchor) {
	scrUser *temp;
	if (delme==NULL) return;
	for (temp=anchor->next; temp->next!=NULL; temp=temp->next) {
		if (temp->next==delme) {
			temp->next=delme->next;
			if (delme->flags != NULL) free(delme->flags);
			if (delme->groups != NULL) free(delme->groups);
			free(delme);
			break;
		}
	}
}
*****************************************************/

int list_is_empty(const scrUser *anchor) {
	return (anchor->next==NULL);
}

scrUser *last_entry(scrUser *anchor) {
	scrUser *temp;
	if (list_is_empty(anchor)) { return anchor; }
/*	temp=anchor->next;
	while (temp->next!=NULL) {
		temp=temp->next;
	}
*/
	for (temp = anchor->next; temp->next != NULL; temp = temp->next);
	return temp;
}

scrUser *find_by_name(const char *name, const scrUser *anchor) {
	scrUser *temp;
	temp=anchor->next;
	while (temp!=NULL) {
		if (strcmp(temp->name,name)==0) return temp;
		temp=temp->next;
	}
	return NULL;
}

void listfree(scrUser *anchor) {
	scrUser *next;
	for(; anchor; anchor=next) {
		next=anchor->next;
		if (free_item(anchor)) printf("Could not free item at address %ld!\n",(long)anchor);
	}
}	
